package jdk.internal.util;

import java.util.function.Function;
import java.util.Collections;
import java.util.Arrays;
import java.util.List;
import java.util.function.BiFunction;

public class Preconditions
{
    public Preconditions() {
    }
    
    private static RuntimeException outOfBounds(BiFunction<String, List<Integer>, ? extends RuntimeException> biFunction, String s, Integer... a) {
        List<Integer> unmodifiableList = Collections.unmodifiableList(Arrays.asList(a));
        RuntimeException ex = (biFunction == null) ? null : ((RuntimeException)biFunction.apply(s, unmodifiableList));
        return (ex == null) ? new IndexOutOfBoundsException(outOfBoundsMessage(s, unmodifiableList)) : ex;
    }
    
    private static RuntimeException outOfBoundsCheckIndex(BiFunction<String, List<Integer>, ? extends RuntimeException> biFunction, int i, int j) {
        return outOfBounds(biFunction, "checkIndex", new Integer[] { Integer.valueOf(i), Integer.valueOf(j) });
    }
    
    private static RuntimeException outOfBoundsCheckFromToIndex(BiFunction<String, List<Integer>, ? extends RuntimeException> biFunction, int i, int j, int k) {
        return outOfBounds(biFunction, "checkFromToIndex", new Integer[] { Integer.valueOf(i), Integer.valueOf(j), Integer.valueOf(k) });
    }
    
    private static RuntimeException outOfBoundsCheckFromIndexSize(BiFunction<String, List<Integer>, ? extends RuntimeException> biFunction, int i, int j, int k) {
        return outOfBounds(biFunction, "checkFromIndexSize", new Integer[] { Integer.valueOf(i), Integer.valueOf(j), Integer.valueOf(k) });
    }
    
    public static <X extends RuntimeException> BiFunction<String, List<Integer>, X> outOfBoundsExceptionFormatter(final Function<String, X> function) {
        return new BiFunction<String, List<Integer>, X>() {
            @Override
            public X apply(String s, List<Integer> list) {
                return (RuntimeException)function.apply(Preconditions.access$000(s, list));
            }
        };
    }
    
    private static String outOfBoundsMessage(String s, List<Integer> list) {
        if (s == null && list == null) {
            return String.format("Range check failed", new Object[0]);
        }
        if (s == null) {
            return String.format("Range check failed: %s", new Object[] { list });
        }
        if (list == null) {
            return String.format("Range check failed: %s", new Object[] { s });
        }
        int n = 0;
        int n2 = -1;
        switch (s.hashCode()) {
            case -538822486: {
                if (s.equals("checkIndex")) {
                    n2 = 0;
                    break;
                }
                break;
            }
            case 1844394469: {
                if (s.equals("checkFromToIndex")) {
                    n2 = 1;
                    break;
                }
                break;
            }
            case 1848935233: {
                if (s.equals("checkFromIndexSize")) {
                    n2 = 2;
                    break;
                }
                break;
            }
        }
        switch (n2) {
            case 0: {
                n = 2;
                break;
            }
            case 1:
            case 2: {
                n = 3;
                break;
            }
        }
        s = ((list.size() != n) ? "" : s);
        int n3 = -1;
        switch (s.hashCode()) {
            case -538822486: {
                if (s.equals("checkIndex")) {
                    n3 = 0;
                    break;
                }
                break;
            }
            case 1844394469: {
                if (s.equals("checkFromToIndex")) {
                    n3 = 1;
                    break;
                }
                break;
            }
            case 1848935233: {
                if (s.equals("checkFromIndexSize")) {
                    n3 = 2;
                    break;
                }
                break;
            }
        }
        switch (n3) {
            case 0: {
                return String.format("Index %d out of bounds for length %d", new Object[] { list.get(0), list.get(1) });
            }
            case 1: {
                return String.format("Range [%d, %d) out of bounds for length %d", new Object[] { list.get(0), list.get(1), list.get(2) });
            }
            case 2: {
                return String.format("Range [%d, %<d + %d) out of bounds for length %d", new Object[] { list.get(0), list.get(1), list.get(2) });
            }
            default: {
                return String.format("Range check failed: %s %s", new Object[] { s, list });
            }
        }
    }
    
    public static <X extends RuntimeException> int checkIndex(int n, int n2, BiFunction<String, List<Integer>, X> biFunction) {
        if (n < 0 || n >= n2) {
            throw outOfBoundsCheckIndex(biFunction, n, n2);
        }
        return n;
    }
    
    public static <X extends RuntimeException> int checkFromToIndex(int n, int n2, int n3, BiFunction<String, List<Integer>, X> biFunction) {
        if (n < 0 || n > n2 || n2 > n3) {
            throw outOfBoundsCheckFromToIndex(biFunction, n, n2, n3);
        }
        return n;
    }
    
    public static <X extends RuntimeException> int checkFromIndexSize(int n, int n2, int n3, BiFunction<String, List<Integer>, X> biFunction) {
        if ((n3 | n | n2) < 0 || n2 > n3 - n) {
            throw outOfBoundsCheckFromIndexSize(biFunction, n, n2, n3);
        }
        return n;
    }
    
    static String access$000(String s, List list) {
        return outOfBoundsMessage(s, list);
    }
}
